@@ -4,7 +4,7 @@ import shutil
 import time
 from contextlib import contextmanager
 from pathlib import Path
-from typing import Callable, Iterator, Optional
+from typing import Callable, Iterator, List, Optional
 
 import pytest
 
@@ -15,6 +15,12 @@ def avoid_identical_ref_timestamps() -> None:
     time.sleep(1.2)
 
 
+def lacks_git_version(version: List[int]) -> bool:
+    if shutil.which("git") is None:
+        return True
+    return _get_git_version() < version
+
+
 REPO = Path(__file__).parent.parent.parent
 
 
@@ -207,6 +213,9 @@ def test__version__from_git__with_annotated_tags(tmp_path) -> None:
             run("git tag v0.3.0+a,b -m Annotated")
             assert from_vcs() == Version("0.3.0", dirty=False, tagged_metadata="a,b", branch=b)
 
+    if not legacy:
+        assert from_vcs(path=vcs) == Version("0.3.0", dirty=False, tagged_metadata="a,b", branch=b)
+
 
 @pytest.mark.skipif(shutil.which("git") is None, reason="Requires Git")
 def test__version__from_git__with_lightweight_tags(tmp_path) -> None:
@@ -451,6 +460,25 @@ def test__version__from_git__shallow(tmp_path) -> None:
             Version.from_git(strict=True)
 
 
+@pytest.mark.skipif(lacks_git_version([2, 27]), reason="Requires Git 2.27+")
+def test__version__from_git__exclude_decoration(tmp_path) -> None:
+    vcs = tmp_path / "dunamai-git-exclude-decoration"
+    vcs.mkdir()
+    run = make_run_callback(vcs)
+    from_vcs = make_from_callback(Version.from_git)
+    b = "master"
+
+    with chdir(vcs):
+        run("git init")
+        (vcs / "foo.txt").write_text("hi")
+        run("git add .")
+        run("git commit --no-gpg-sign -m Initial")
+        run("git tag v0.1.0 -m Release")
+        run("git config log.excludeDecoration refs/tags/")
+
+        assert from_vcs() == Version("0.1.0", dirty=False, branch=b)
+
+
 @pytest.mark.skipif(shutil.which("git") is None, reason="Requires Git")
 def test__version__not_a_repository(tmp_path) -> None:
     vcs = tmp_path / "dunamai-not-a-repo"
@@ -518,6 +546,8 @@ def test__version__from_mercurial(tmp_path) -> None:
         assert from_vcs() == Version("0.1.0", dirty=False, branch=b)
         assert from_vcs(latest_tag=True) == Version("0.1.0", dirty=False, branch=b)
 
+    assert from_vcs(path=vcs) == Version("0.1.0", dirty=False, branch=b)
+
 
 def test__version__from_mercurial__archival_untagged() -> None:
     with chdir(REPO / "tests" / "archival" / "hg-untagged"):
@@ -591,6 +621,8 @@ def test__version__from_darcs(tmp_path) -> None:
         run("darcs obliterate --all --last 3")
         assert from_vcs() == Version("0.1.0", dirty=False)
 
+    assert from_vcs(path=vcs) == Version("0.1.0", dirty=False)
+
 
 @pytest.mark.skipif(
     None in [shutil.which("svn"), shutil.which("svnadmin")], reason="Requires Subversion"
@@ -687,6 +719,8 @@ def test__version__from_subversion(tmp_path) -> None:
                 "0.1.0", distance=0, commit="3", dirty=False
             )
 
+    assert from_vcs(path=vcs) == Version("0.1.0", distance=0, commit="3", dirty=False)
+
 
 @pytest.mark.skipif(shutil.which("bzr") is None, reason="Requires Bazaar")
 def test__version__from_bazaar(tmp_path) -> None:
@@ -751,6 +785,8 @@ def test__version__from_bazaar(tmp_path) -> None:
         run("bzr tag v0.2.1")
         assert from_vcs() == Version("0.2.1", commit="3", dirty=False, branch="renamed")
 
+    assert from_vcs(path=vcs) == Version("0.2.1", commit="3", dirty=False, branch="renamed")
+
 
 @pytest.mark.skipif(shutil.which("fossil") is None, reason="Requires Fossil")
 def test__version__from_fossil(tmp_path) -> None:
@@ -806,6 +842,8 @@ def test__version__from_fossil(tmp_path) -> None:
         assert from_vcs() == Version("0.1.1", dirty=False, branch=b)
         assert from_vcs(latest_tag=True) == Version("0.1.1", dirty=False, branch=b)
 
+    assert from_vcs(path=vcs) == Version("0.1.1", dirty=False, branch=b)
+
 
 @pytest.mark.skipif(shutil.which("pijul") is None, reason="Requires Pijul")
 def test__version__from_pijul(tmp_path) -> None:
@@ -845,3 +883,5 @@ def test__version__from_pijul(tmp_path) -> None:
         assert from_vcs() == Version("0.1.0", distance=1, dirty=False, branch=b)
         with pytest.raises(ValueError):
             from_vcs(latest_tag=True)
+
+    assert from_vcs(path=vcs) == Version("0.1.0", distance=1, dirty=False, branch=b)
